<?php

if( ! function_exists( 'mc_ext_ecwid_api' ) ) {
	function mc_ext_ecwid_api() {
		if( ! class_exists( 'EcwidProductApi' ) ) {
			include_once WP_PLUGIN_DIR . '/ecwid-shopping-cart/lib/ecwid_product_api.php';
		}

		$store_id = get_ecwid_store_id();

		return new EcwidProductApi( $store_id );
	}
}

if( ! function_exists( 'mc_ext_ecwid_product_html' ) ) {
	function mc_ext_ecwid_product_html( $product = array() ) {

		if( ! empty( $product ) ) {
			$ecwid_api = mc_ext_ecwid_api();
			if( is_array( $product ) ) {
				$product_id = $product['id'];
				$product_url = isset( $product['url'] ) ? $product['url'] : '';
				$product_name = isset( $product['name'] ) ? $product['name'] : '';
				$product_thumbnail_url = isset( $product['thumbnailUrl'] ) ? $product['thumbnailUrl'] : '';
				$product_favorite_count = isset( $product['favorites']['displayedCount'] ) ? $product['favorites']['displayedCount'] : 0;
				$product_price = isset( $product['price'] ) ? $product['price'] : 0;
			} else {
				$product_id = $product->id;
				$product_url = isset( $product->url ) ? $product->url : '';
				$product_name = isset( $product->name ) ? $product->name : '';
				$product_thumbnail_url = isset( $product->thumbnailUrl ) ? $product->thumbnailUrl : '';
				$product_favorite_count = isset( $product->favorites->displayedCount ) ? $product->favorites->displayedCount : 0;
				$product_price = isset( $product->price ) ? $product->price : 0;
			}
			$product = $ecwid_api->get_product( $product_id );
			$profile = $ecwid_api->get_profile();

			if( is_array( $product ) ) {
				$product_taxes = isset( $product['taxes'] ) ? $product['taxes'] : array();
			} else {
				$product_taxes = isset( $product->taxes ) ? $product->taxes : array();
			}
			
			ob_start();
			?>
			<div class="product">
				<div class="product-inner">		
					<a href="<?php echo ecwid_get_product_url( array( 'id' => $product_id, 'url' => $product_url ) ); ?>">
						<div class="ecwid-favorite">
							<span><?php echo $product_favorite_count; ?></span>
						</div>

						<div class="product-thumbnail-wrapper">
							<?php if( ! empty( $product_thumbnail_url ) ) : ?>
								<img class="attachment-shop_catalog wp-post-image" width="246" height="186" src="<?php echo $product_thumbnail_url; ?>" alt="">
							<?php endif; ?>
						</div>

						<div class="title-area">
							<h3><?php echo $product_name; ?></h3>
						</div>

						<span class="price">
							<span class="mc-price-wrapper">
								<?php $product_price_with_tax = mc_ext_ecwid_product_tax_price( $product_price, $product_taxes ); ?>
								<span class="amount"><?php echo $profile['currencyPrefix'] . mc_ext_ecwid_product_formated_price( $product_price_with_tax, $profile['numericPrecision'], $profile['decimalSeparator'], $profile['groupSeparator'] ) . $profile['currencySuffix']; ?></span>
							</span>
						</span>
					</a>				
				</div>
			</div>
			<?php
			$html = ob_get_clean();
			return $html;
		}

		return;
	}
}

if( ! function_exists( 'mc_ext_ecwid_product_tax_price' ) ) {
	function mc_ext_ecwid_product_tax_price( $price = 0, $tax_arr = array() ) {
		
		$new_price = $price;

		$tax_percetage = 0;
		if( ! empty( $tax_arr ) ) {
			foreach ( $tax_arr as $key => $tax ) {
				$tax_value = 0;
				if( is_array( $tax ) ) {
					$tax_value = isset( $tax['value'] ) ? $tax['value'] : 0;
				} else {
					$tax_value = isset( $tax->value ) ? $tax->value : 0;
				}
				
				if( $tax_value > 0 ) {
					$tax_percetage += $tax_value;
				}
			}
		}

		if( $tax_percetage > 0 ) {
			$tax_amount = $price * ( $tax_percetage / 100 );
			$new_price = $price + $tax_amount;
		}

		return $new_price;
	}
}

if( ! function_exists( 'mc_ext_ecwid_product_formated_price' ) ) {
	function mc_ext_ecwid_product_formated_price( $price, $numeric_precision, $decimal_separator, $group_separator ) {
		return number_format( $price, $numeric_precision, $decimal_separator, $group_separator );
	}
}